// Ubiquity provides a standards-based suite of browser enhancements for
// building a new generation of internet-related applications.
//
// The Ubiquity SMIL module adds SMIL support to the Ubiquity library.
//
// Copyright (C) 2008 Backplane Ltd.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

function BlinkBlink(elmnt)
{
	this.element = elmnt;
	this.rate = 0.5;	// How many seconds per blink cycle
	this.count = Infinity;	// How many blink cycles
	this.curState = 1;		// 1 for visible, 0 for invisible
	this.states = ["hidden", "visible"];
	this.callbackID = undefined;
	this.blinkBeginRenderer();
}

BlinkBlink.prototype.blinkBeginRenderer = function()
{
	var rate = this.element.getAttribute("rate");
	var count = this.element.getAttribute("count");
	if (this.element.style.visibility == "hidden") this.curState = 0;

	// Adapt rate and count: 10 1second cycles are 20 0.5second half-cycles
	if (rate != undefined) this.rate = rate / 2;
	if (count != undefined) this.count = count * 2;

	this.blinkAction();

	var that = this;
	function callback() {
		that.count--;
		if (that.count <= 0) {
			that.blinkEndRenderer();
		} else {
			that.curState = 1 - that.curState;
			that.blinkAction();
		}
	}
	this.callbackID = setInterval(callback, this.rate*1000);
	
}

BlinkBlink.prototype.blinkEndRenderer = function()
{
	this.count = 0;
	if (this.callbackID != undefined)
		clearInterval(this.callbackID);
}

BlinkBlink.prototype.blinkAction = function()
{
	this.element.style.visibility = this.states[this.curState];
}

/*
BlinkBlink.prototype.handleEvent = function(evt)
{
	var rate = this.element.getAttribute("rate");
	var count = this.element.getAttribute("count");
}
*/

/*BlinkBlink.prototype.ctor = function()
{
	alert(this.element.banana);
}*/