// Ubiquity provides a standards-based suite of browser enhancements for
// building a new generation of internet-related applications.
//
// The Ubiquity SMIL module adds SMIL support to the Ubiquity library.
//
// Copyright (C) 2008 Backplane Ltd.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/**@fileoverview 
	Contains the definitions used to set up the decorator for SMIL.
*/

/*global DECORATOR, NamespaceManager*/
/*global SmilSmil, SmilBody*/
/*global SmilLayout, SmilTopLayout, SmilRegion*/
/*global SmilPlayableAnimation, SmilPlayableAudio, SmilPlayableImg*/
/*global SmilPlayableRef, SmilPlayableText, SmilPlayableTextstream*/
/*global SmilPlayableVideo*/
/*global SmilSchedMedia, SmilSchedPar, SmilSchedSeq*/

/**@requires
	decorate.js
*/

(
  function() {
	// Add XForms decoration rules
	DECORATOR.addDecorationRules({
		"namespaceURI" : "http://www.w3.org/ns/SMIL",
		"rules" : {
			// Structure module
			"smil" : [
			{
				"name" : "smil-smil-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilSmil]);
				}
			}
			],
			"head" : [
			{
				"name" : "smil-head-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([]);
				}
			}
			],
			"body" : [
			{
				"name" : "smil-body-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilSchedSeq, SmilBody]);
				}
			}
			],
			// Layout module
			"layout" : [
			{
				"name" : "smil-layout-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilLayout]);
				}
			}
			],
			"toplayout" : [
			{
				"name" : "smil-toplayout-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilTopLayout]);
				}
			}
			],
			"root-layout" : [
			{
				"name" : "smil-root-layout-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilTopLayout]);
				}
			}
			],
			"region" : [
			{
				"name" : "smil-region-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilRegion]);
				}
			}
			],
			// Metainformation module
			"meta" : [
			{
				"name" : "smil-meta-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([]);
				}
			}
			],
			"metadata" : [
			{
				"name" : "smil-metadata-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([]);
				}
			}
			],
			// Media Object module
			"ref" : [
			{
				"name" : "smil-ref-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilPlayableRef, SmilSchedMedia]);
				}
			}
			],
			"animation" : [
			{
				"name" : "smil-animation-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilPlayableAnimation, SmilSchedMedia]);
				}
			}
			],
			"audio" : [
			{
				"name" : "smil-audio-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilPlayableAudio, SmilSchedMedia]);
				}
			}
			],
			"img" : [
			{
				"name" : "smil-img-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilPlayableImg, SmilSchedMedia]);
				}
			}
			],
			"text" : [
			{
				"name" : "smil-text-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilPlayableText, SmilSchedMedia]);
				}
			}
			],
			"textstream" : [
			{
				"name" : "smil-textstream-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilPlayableTextstream, SmilSchedMedia]);
				}
			}
			],
			"video" : [
			{
				"name" : "smil-video-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilPlayableVideo, SmilSchedMedia]);
				}
			}
			],
			// Timing and Synchronization module
			"par" : [
			{
				"name" : "smil-par-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilSchedPar]);
				}
			}
			],
			"seq" : [
			{
				"name" : "smil-seq-element",
				"apply" : function(arrBehaviours) {
					return arrBehaviours.concat([SmilSchedSeq]);
				}
			}
			]
		}
	});
	
	NamespaceManager.addSelectionNamespace("smil", "http://www.w3.org/ns/SMIL");	
	
	DECORATOR.setupDecorator(
		[
			{
				selector: "smil|smil",
				objects: [  ]
			},
			{
				selector: "smil|head",
				objects: [  ]
			},
			{
				selector: "smil|body",
				objects: [  ]
			},
			{
				selector: "smil|layout",
				objects: [  ]
			},
			{
				selector: "smil|toplayout",
				objects: [  ]
			},
			{
				selector: "smil|root-layout",
				objects: [  ]
			},
			{
				selector: "smil|region",
				objects: [  ]
			},
			{
				selector: "smil|meta",
				objects: [  ]
			},
			{
				selector: "smil|metadata",
				objects: [  ]
			},
			{
				selector: "smil|ref",
				objects: [  ]
			},
			{
				selector: "smil|animation",
				objects: [  ]
			},
			{
				selector: "smil|audio",
				objects: [  ]
			},
			{
				selector: "smil|img",
				objects: [  ]
			},
			{
				selector: "smil|text",
				objects: [  ]
			},
			{
				selector: "smil|textstream",
				objects: [  ]
			},
			{
				selector: "smil|video",
				objects: [  ]
			},
			{
				selector: "smil|par",
				objects: [  ]
			},
			{
				selector: "smil|seq",
				objects: [  ]
			}
	
		],
		"http://www.w3.org/ns/SMIL");
  } ()
);
